



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
jupyter server -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
echo "not (delete or merge_config or test_culling or test_restart_kernel[jp_server_config0] or test_connection[jp_server_config0] or test_terminal or test_authorized_requests or test_bad_symlink or test_good_symlink)"
IF %ERRORLEVEL% NEQ 0 exit /B 1
pytest -vvv --cov=jupyter_server -k "not (delete or merge_config or test_culling or test_restart_kernel[jp_server_config0] or test_connection[jp_server_config0] or test_terminal or test_authorized_requests or test_bad_symlink or test_good_symlink)" --cov-report=term-missing:skip-covered --no-cov-on-fail --cov-fail-under=70
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
